@@ -156,14 +156,10 @@ void                       ephy_web_view_set_typed_address        (EphyWebView
                                                                    const char                *address);
 gboolean                   ephy_web_view_get_is_blank             (EphyWebView               *view);
 gboolean                   ephy_web_view_has_modified_forms       (EphyWebView               *view);
-char *                     ephy_web_view_get_location             (EphyWebView               *view,
-                                                                   gboolean                   toplevel);
 void                       ephy_web_view_get_security_level       (EphyWebView               *view,
                                                                    EphyWebViewSecurityLevel  *level,
                                                                    char                     **description);
 void                       ephy_web_view_print                    (EphyWebView               *view);
-void                       ephy_web_view_set_title                (EphyWebView               *view,
-                                                                   const char                *view_title);
 const char *               ephy_web_view_get_title                (EphyWebView               *view);
 const char *               ephy_web_view_get_address              (EphyWebView               *view);
 const char *               ephy_web_view_get_title_composite      (EphyWebView               *view);
@@ -172,28 +168,6 @@ void                       ephy_web_view_load_error_page          (EphyWebView
                                                                    const char                *uri,
                                                                    EphyWebViewErrorPage       page,
                                                                    GError                    *error);
-/* These should be private */
-void                       ephy_web_view_set_address              (EphyWebView               *view,
-                                                                   const char                *address);
-void                       ephy_web_view_location_changed         (EphyWebView               *view,
-                                                                   const char                *location);
-void                       ephy_web_view_set_loading_title        (EphyWebView               *view,
-                                                                   const char                *title,
-                                                                   gboolean                   is_address);
-void                       ephy_web_view_popups_manager_reset     (EphyWebView               *view);
-void                       ephy_web_view_save                     (EphyWebView               *view,
-                                                                   const char                *uri);
-void                       ephy_web_view_load_homepage            (EphyWebView               *view);
-
-char *                     ephy_web_view_create_web_application   (EphyWebView               *view,
-                                                                   const char                *title,
-                                                                   GdkPixbuf                 *icon);
-GdkPixbuf *                ephy_web_view_get_snapshot             (EphyWebView               *view,
-                                                                   int                        x,
-                                                                   int                        y,
-                                                                   int                        width,
-                                                                   int                        height);
-gboolean                   ephy_web_view_is_loading_homepage      (EphyWebView               *view);
 
 G_END_DECLS
 
