@@ -30,9 +30,6 @@
 #include <gio/gio.h>
 #include <gtk/gtk.h>
 
-/* From libgnome */
-#define GNOME_DOT_GNOME ".gnome2"
-
 extern GQuark ephy_file_helpers_error_quark;
 #define EPHY_FILE_HELPERS_ERROR_QUARK	(ephy_file_helpers_error_quark)
 
@@ -45,11 +42,18 @@ typedef enum
 	EPHY_MIME_PERMISSION_UNKNOWN	= 3
 } EphyMimePermission;
 
+typedef enum
+{
+	EPHY_FILE_HELPERS_NONE		   = 0,
+	EPHY_FILE_HELPERS_KEEP_TEMP_DIR	   = 1 << 1,
+	EPHY_FILE_HELPERS_PRIVATE_PROFILE  = 1 << 2,
+	EPHY_FILE_HELPERS_ENSURE_EXISTS	   = 1 << 3,
+} EphyFileHelpersFlags;
+
 #define EPHY_UUID_ENVVAR	"EPHY_UNIQUE"
 
 gboolean           ephy_file_helpers_init        (const char  *profile_dir,
-                                                  gboolean     private_profile,
-                                                  gboolean     keep_temp_dir,
+                                                  EphyFileHelpersFlags flags,
                                                   GError     **error);
 const char *       ephy_file                     (const char  *filename);
 const char *       ephy_dot_dir                  (void);
@@ -84,8 +88,8 @@ gboolean           ephy_file_browse_to           (GFile       *file,
 gboolean           ephy_file_delete_dir_recursively (GFile *file,
                                                      GError      **error);
 void               ephy_file_delete_uri          (const char  *uri);
-void               ephy_file_load_accels (void);
-void               ephy_file_save_accels (void);
+char       *       ephy_file_create_data_uri_for_filename (const char *filename,
+                                                           const char *mime_type);
 
 G_END_DECLS
 
