@@ -37,14 +37,20 @@
 #include "ephy-history-service.h"
 #include "ephy-profile-utils.h"
 #include "ephy-settings.h"
+#include "ephy-web-app-utils.h"
 #ifdef ENABLE_NSS
 #include "ephy-nss-glue.h"
 #endif
 
+#include <fcntl.h>
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 #include <gnome-keyring.h>
 #include <libsoup/soup-gnome.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+
+static int do_step_n = -1;
 
 /*
  * What to do to add new migration steps:
@@ -533,16 +539,6 @@ static GMarkupParser history_parse_funcs =
   NULL,
 };
 
-static EphyHistoryService *
-ensure_empty_history (const char* filename)
-{
-  if (g_file_test (filename, G_FILE_TEST_IS_REGULAR)) {
-    g_unlink (filename);
-  }
-
-  return ephy_history_service_new (filename);
-}
-
 static void
 migrate_history ()
 {
@@ -555,7 +551,15 @@ migrate_history ()
   HistoryParseData parse_data;
 
   gchar *temporary_file = g_build_filename (ephy_dot_dir (), "ephy-history.db", NULL);
-  history_service = ensure_empty_history (temporary_file);
+  /* Do nothing if the history file already exists. Safer than wiping
+   * it out. */
+  if (g_file_test (temporary_file, G_FILE_TEST_EXISTS)) {
+    g_warning ("Did not migrate Epiphany's history, the ephy-history.db file already exists");
+    g_free (temporary_file);
+    return;
+  }
+
+  history_service = ephy_history_service_new (temporary_file);
   g_free (temporary_file);
 
   memset (&parse_data, 0, sizeof (HistoryParseData));
@@ -620,6 +624,145 @@ migrate_tabs_visibility ()
                          EPHY_PREFS_UI_TABS_BAR_VISIBILITY_POLICY_ALWAYS);
 }
 
+static void
+migrate_profile (const char *old_dir,
+                 const char *new_dir)
+{
+  char *parent_dir;
+  char *updated;
+  const char *message;
+
+  if (g_file_test (new_dir, G_FILE_TEST_EXISTS) ||
+      !g_file_test (old_dir, G_FILE_TEST_IS_DIR))
+    return;
+
+  /* Test if we already attempted to migrate first. */
+  updated = g_build_filename (old_dir, "DEPRECATED-DIRECTORY", NULL);
+  message = _("Epiphany 3.6 deprecated this directory and tried migrating "
+              "this configuration to ~/.config/epiphany");
+
+  parent_dir = g_path_get_dirname (new_dir);
+  if (g_mkdir_with_parents (parent_dir, 0700) == 0) {
+    int fd, res;
+
+    /* rename() works fine if the destination directory is empty. */
+    res = g_rename (old_dir, new_dir);
+    if (res == -1 && !g_file_test (updated, G_FILE_TEST_EXISTS)) {
+      fd = g_creat (updated, 0600);
+      if (fd != -1) {
+        res = write (fd, message, strlen (message));
+        close (fd);
+      }
+    }
+  }
+
+  g_free (parent_dir);
+  g_free (updated);
+}
+
+static void
+migrate_profile_gnome2_to_xdg ()
+{
+  char *old_dir;
+  char *new_dir;
+
+  old_dir = g_build_filename (g_get_home_dir (),
+                              ".gnome2",
+                              "epiphany",
+                              NULL);
+  new_dir = g_build_filename (g_get_user_config_dir (),
+                              "epiphany",
+                              NULL);
+
+  migrate_profile (old_dir, new_dir);
+
+  g_free (new_dir);
+  g_free (old_dir);
+}
+
+static char *
+fix_desktop_file_and_return_new_location (const char *dir)
+{
+  GRegex * regex;
+  char *result, *old_profile_dir, *replacement, *contents, *new_contents;
+  gsize length;
+
+  old_profile_dir = g_build_filename (g_get_home_dir (),
+                                      ".gnome2",
+                                      NULL);
+  replacement = g_build_filename (g_get_user_config_dir (),
+                                  NULL);
+  regex = g_regex_new (old_profile_dir, 0, 0, NULL);
+
+  /* We want to modify both the link destination and the contents of
+   * the .desktop file itself. */
+  result = g_regex_replace (regex, dir, -1,
+                            0, replacement, 0, NULL);
+  g_file_get_contents (result, &contents, &length, NULL);
+  new_contents = g_regex_replace (regex, contents, -1, 0,
+                                  replacement, 0, NULL);
+  g_file_set_contents (result, new_contents, length, NULL);
+
+  g_free (contents);
+  g_free (new_contents);
+  g_free (old_profile_dir);
+  g_free (replacement);
+
+  g_regex_unref (regex);
+
+  return result;
+}
+
+static void
+migrate_web_app_links ()
+{
+  GList *apps, *p;
+
+  apps = ephy_web_application_get_application_list ();
+  for (p = apps; p; p = p->next) {
+    char *desktop_file, *app_link;
+    EphyWebApplication *app = (EphyWebApplication*)p->data;
+
+    desktop_file = app->desktop_file;
+
+    /* Update the link in applications. */
+    app_link = g_build_filename (g_get_user_data_dir (), "applications", desktop_file, NULL);
+    if (g_file_test (app_link, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_SYMLINK)) {
+        /* Change the link to point to the new profile dir. */
+        GFileInfo *info;
+        const char *target;
+        GFile *file = g_file_new_for_path (app_link);
+        
+        info = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_SYMLINK_TARGET,
+                                  0, NULL, NULL);
+        if (info) {
+          char *new_target;
+
+          target = g_file_info_get_symlink_target (info);
+          new_target = fix_desktop_file_and_return_new_location (target);
+
+          /* FIXME: Updating the file info and setting it again should
+           * work, but it does not? Just delete and create the link
+           * again. */
+          g_file_delete (file, 0, 0);
+          g_object_unref (file);
+
+          file = g_file_new_for_path (app_link);
+          g_file_make_symbolic_link (file, new_target, NULL, NULL);
+
+          g_object_unref (info);
+          g_free (new_target);
+        }
+
+        g_object_unref (file);
+    }
+
+    g_free (app_link);
+  }
+
+  ephy_web_application_free_application_list (apps);
+}
+
 const EphyProfileMigrator migrators[] = {
   migrate_cookies,
   migrate_passwords,
@@ -630,13 +773,30 @@ const EphyProfileMigrator migrators[] = {
    * login/passwords for page forms, which we previously ignored */
   migrate_passwords2,
   migrate_history,
-  migrate_tabs_visibility
+  migrate_tabs_visibility,
+  migrate_web_app_links
 };
 
-static void
+static gboolean
 ephy_migrator ()
 {
   int latest, i;
+  EphyProfileMigrator m;
+
+  /* Always try to migrate the data from the old profile dir at the
+   * very beginning. */
+  migrate_profile_gnome2_to_xdg ();
+
+  if (do_step_n != -1) {
+    if (do_step_n >= EPHY_PROFILE_MIGRATION_VERSION)
+      return FALSE;
+
+    LOG ("Running only migrator: %d", do_step_n);
+    m = migrators[do_step_n];
+    m();
+
+    return TRUE;
+  }
 
   latest = ephy_profile_utils_get_migration_version ();
 
@@ -644,8 +804,6 @@ ephy_migrator ()
        EPHY_PROFILE_MIGRATION_VERSION, latest);
 
   for (i = latest; i < EPHY_PROFILE_MIGRATION_VERSION; i++) {
-    EphyProfileMigrator m;
-
     LOG ("Running migrator: %d of %d", i, EPHY_PROFILE_MIGRATION_VERSION);
 
     /* No need to run the password migration twice in a row. It
@@ -659,23 +817,57 @@ ephy_migrator ()
     m();
   }
 
-  if (ephy_profile_utils_set_migration_version (EPHY_PROFILE_MIGRATION_VERSION) != TRUE)
+  if (ephy_profile_utils_set_migration_version (EPHY_PROFILE_MIGRATION_VERSION) != TRUE) {
     LOG ("Failed to store the current migration version");
+    return FALSE;
+  }
+
+  return TRUE;
 }
 
+static const GOptionEntry option_entries[] =
+{
+  { "do-step", 'd', 0, G_OPTION_ARG_INT, &do_step_n,
+    N_("Executes only the n-th migration step"), NULL },
+  { NULL }
+};
+
 int
 main (int argc, char *argv[])
 {
+  GOptionContext *option_context;
+  GOptionGroup *option_group;
+  GError *error = NULL;
+
   g_type_init ();
 
+  option_group = g_option_group_new ("ephy-profile-migrator",
+                                     N_("Epiphany profile migrator"),
+                                     N_("Epiphany profile migrator options"),
+                                     NULL, NULL);
+
+  g_option_group_set_translation_domain (option_group, GETTEXT_PACKAGE);
+  g_option_group_add_entries (option_group, option_entries);
+
+  option_context = g_option_context_new ("");
+  g_option_context_set_main_group (option_context, option_group);
+
+  if (!g_option_context_parse (option_context, &argc, &argv, &error)) {
+    g_print ("Failed to parse arguments: %s\n", error->message);
+    g_error_free (error);
+    g_option_context_free (option_context);
+
+    return 1;
+  }
+        
+  g_option_context_free (option_context);
+
   ephy_debug_init ();
 
-  if (!ephy_file_helpers_init (NULL, FALSE, FALSE, NULL)) {
+  if (!ephy_file_helpers_init (NULL, EPHY_FILE_HELPERS_NONE, NULL)) {
     LOG ("Something wrong happened with ephy_file_helpers_init()");
     return -1;
   }
 
-  ephy_migrator ();
-
-  return 0;
+  return ephy_migrator () ? 0 : 1;
 }
