@@ -20,26 +20,26 @@
  */
 
 #include "config.h"
-
 #include "ephy-bookmarks.h"
-#include "ephy-bookmarks-type-builtins.h"
-#include "ephy-file-helpers.h"
-#include "ephy-embed-shell.h"
-#include "ephy-shell.h"
-#include "ephy-history.h"
-#include "ephy-debug.h"
-#include "ephy-tree-model-node.h"
-#include "ephy-node-common.h"
+
+#include "ephy-bookmark-properties.h"
 #include "ephy-bookmarks-export.h"
 #include "ephy-bookmarks-import.h"
-#include "ephy-bookmark-properties.h"
+#include "ephy-bookmarks-type-builtins.h"
+#include "ephy-debug.h"
+#include "ephy-embed-shell.h"
+#include "ephy-file-helpers.h"
+#include "ephy-history-service.h"
+#include "ephy-node-common.h"
 #include "ephy-prefs.h"
 #include "ephy-settings.h"
+#include "ephy-shell.h"
 #include "ephy-signal-accumulator.h"
+#include "ephy-tree-model-node.h"
 
-#include <string.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#include <string.h>
 
 #ifdef ENABLE_ZEROCONF
 #include <avahi-common/error.h>
@@ -52,7 +52,6 @@
 #define EPHY_BOOKMARKS_XML_ROOT    "ephy_bookmarks"
 #define EPHY_BOOKMARKS_XML_VERSION "1.03"
 #define BOOKMARKS_SAVE_DELAY 3 /* seconds */
-#define MAX_FAVORITES_NUM 10
 #define UPDATE_URI_DATA_KEY "updated-uri"
 
 #define EPHY_BOOKMARKS_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_BOOKMARKS, EphyBookmarksPrivate))
@@ -74,7 +73,6 @@ struct _EphyBookmarksPrivate
 	EphyNodeDb *db;
 	EphyNode *bookmarks;
 	EphyNode *keywords;
-	EphyNode *favorites;
 	EphyNode *notcategorized;
 	EphyNode *smartbookmarks;
 	EphyNode *lower_fav;
@@ -173,7 +171,6 @@ save_filter (EphyNode *node,
 	EphyBookmarksPrivate *priv = bookmarks->priv;
 
 	return node != priv->bookmarks &&
-	       node != priv->favorites &&
 	       node != priv->notcategorized &&
 #ifdef ENABLE_ZEROCONF
 	       node != priv->local;
@@ -250,247 +247,34 @@ ephy_bookmarks_save_delayed (EphyBookmarks *bookmarks, int delay)
 	}
 }
 
-static double
-get_history_item_score (EphyHistory *eh, const char *page)
-{
-	return ephy_history_get_page_visits (eh, page);
-}
-
-static EphyNode *
-compute_lower_fav (EphyNode *favorites, double *score)
-{
-	GPtrArray *children;
-	int i;
-	EphyHistory *history;
-	EphyNode *result = NULL;
-
-	history = EPHY_HISTORY (ephy_embed_shell_get_global_history (embed_shell));
-
-	*score = DBL_MAX;
-	children = ephy_node_get_children (favorites);
-	for (i = 0; i < children->len; i++)
-	{
-		const char *url;
-		EphyNode *child;
-		double item_score;
-
-		child = g_ptr_array_index (children, i);
-		url = ephy_node_get_property_string
-			(child, EPHY_NODE_BMK_PROP_LOCATION);
-		item_score = get_history_item_score (history, url);
-		if (*score > item_score)
-		{
-			*score = item_score;
-			result = child;
-		}
-	}
-
-	if (result == NULL) *score = 0;
-
-	return result;
-}
-
+#ifdef HAVE_WEBKIT2
+/* TODO: Favicons */
+#else
 static void
-ephy_bookmarks_update_favorites (EphyBookmarks *eb)
-{
-	eb->priv->lower_fav = compute_lower_fav (eb->priv->favorites,
-						 &eb->priv->lower_score);
-}
-
-static gboolean
-add_to_favorites (EphyBookmarks *bookmarks,
-		  EphyNode *node,
-		  EphyHistory *history)
-{
-	EphyBookmarksPrivate *priv = bookmarks->priv;
-	const char *url;
-	gboolean full_menu;
-	double score;
-
-	if (ephy_node_db_is_immutable (priv->db)) return FALSE;
-	if (ephy_node_has_child (priv->favorites, node)) return FALSE;
-
-	url = ephy_node_get_property_string (node, EPHY_NODE_BMK_PROP_LOCATION);
-	score = get_history_item_score (history, url);
-	full_menu = ephy_node_get_n_children (priv->favorites)
-		    >= MAX_FAVORITES_NUM;
-	if (full_menu && score < priv->lower_score) return FALSE;
-
-	if (priv->lower_fav && full_menu)
-	{
-		ephy_node_remove_child (priv->favorites,
-					priv->lower_fav);
-	}
-
-	ephy_node_add_child (priv->favorites, node);
-	ephy_bookmarks_update_favorites (bookmarks);
-
-	return TRUE;
-}
-
-static void
-history_site_visited_cb (EphyHistory *history,
-			 const char *url,
-			 EphyBookmarks *bookmarks)
-{
-	EphyNode *node;
-
-	node = ephy_bookmarks_find_bookmark (bookmarks, url);
-	if (node == NULL) return;
-
-	add_to_favorites (bookmarks, node, history);
-}
-
-static void
-clear_favorites (EphyBookmarks *bookmarks)
-{
-	EphyNode *node;
-	GPtrArray *children;
-	int i;
-	gboolean was_immutable;
-
-	/* clear the favourites */
-
-	was_immutable = ephy_node_db_is_immutable (bookmarks->priv->db);
-	ephy_node_db_set_immutable (bookmarks->priv->db, FALSE);
-
-	node = bookmarks->priv->favorites;
-	children = ephy_node_get_children (node);
-	for (i = (int) children->len - 1; i >= 0; i--)
-	{
-		EphyNode *kid;
-
-		kid = g_ptr_array_index (children, i);
-
-		ephy_node_remove_child (node, kid);
-	}
-
-	ephy_node_db_set_immutable (bookmarks->priv->db, was_immutable);
-
-	ephy_bookmarks_update_favorites (bookmarks);
-}
-
-static void
-history_cleared_cb (EphyHistory *history,
-		    EphyBookmarks *bookmarks)
-{
-	clear_favorites (bookmarks);
-}
-
-static void
-update_bookmark_response_cb (GtkWidget *dialog,
-			     int response,
-			     EphyNode *bookmark)
-{
-	GValue value = { 0, };
-	char *to_uri;
-
-	if (response == GTK_RESPONSE_ACCEPT)
-	{
-		to_uri = (char *) g_object_steal_data (G_OBJECT (dialog),
-						       UPDATE_URI_DATA_KEY);
-		
-		g_value_init (&value, G_TYPE_STRING);
-		g_value_take_string (&value, to_uri);
-		ephy_node_set_property (bookmark, EPHY_NODE_BMK_PROP_LOCATION,
-					&value);
-		g_value_unset (&value);
-	}
-
-	gtk_widget_destroy (dialog);
-}
-
-static void
-update_bookmark_destroy_cb (EphyNode *zombie,
-			    GtkWidget *dialog)
-{
-	gtk_widget_destroy (dialog);
-}
-
-static void
-redirect_cb (EphyHistory *history,
-	      const char *from_uri,
-	      const char *to_uri,
-	      EphyBookmarks *eb)
-{
-	EphyNode *bookmark;
-	GtkWidget *dialog;
-	const char *title;
-
-	bookmark = ephy_bookmarks_find_bookmark (eb, from_uri);
-
-	/* FIXME check if there's another update-bookmark dialog up
-	 * for from_uri' -> from_uri and update it accordingly
-	 */
-	if (bookmark == NULL) return;
-
-	title = ephy_node_get_property_string (bookmark, EPHY_NODE_BMK_PROP_TITLE);
-
-	dialog = gtk_message_dialog_new
-			(NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
-				GTK_MESSAGE_QUESTION,
-				GTK_BUTTONS_NONE,
-				/* translators: the %s is the title of the bookmark */
-				_("Update bookmark “%s”?"),
-				title);
-	gtk_message_dialog_format_secondary_text
-		(GTK_MESSAGE_DIALOG (dialog),
-		/* translators: the %s is a URL */
-		_("The bookmarked page has moved to “%s”."),
-		to_uri);
-
-	gtk_dialog_add_button (GTK_DIALOG (dialog),
-				_("_Don't Update"), GTK_RESPONSE_REJECT);
-	gtk_dialog_add_button (GTK_DIALOG (dialog),
-				_("_Update"), GTK_RESPONSE_ACCEPT);
-	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_REJECT);
-
-	gtk_window_set_title (GTK_WINDOW (dialog), _("Update Bookmark?"));
-
-	g_object_set_data_full (G_OBJECT (dialog), UPDATE_URI_DATA_KEY,
-				g_strdup (to_uri), (GDestroyNotify) g_free);
-	g_signal_connect (dialog, "response",
-			  G_CALLBACK (update_bookmark_response_cb), bookmark);
-	ephy_node_signal_connect_object (bookmark,
-					 EPHY_NODE_DESTROY,
-					 (EphyNodeCallback) update_bookmark_destroy_cb,
-					 G_OBJECT (dialog));
-
-	/* this dialogue is unexpected */
-	gtk_window_set_focus_on_map (GTK_WINDOW (dialog), FALSE);
-
-	gtk_window_present (GTK_WINDOW (dialog));
-}
-
-static void
-icon_updated_cb (EphyHistory *history,
+icon_updated_cb (WebKitFaviconDatabase *favicon_database,
 		 const char *address,
-		 const char *icon,
 		 EphyBookmarks *eb)
 {
+	char *icon;
+
+	icon = webkit_favicon_database_get_favicon_uri (favicon_database, address);
 	ephy_bookmarks_set_icon (eb, address, icon);
+	g_free (icon);
 }
+#endif
 
 static void
 ephy_setup_history_notifiers (EphyBookmarks *eb)
 {
-	EphyHistory *history;
-
-	history = EPHY_HISTORY (ephy_embed_shell_get_global_history (embed_shell));
-
-	if (ephy_history_is_enabled (history) == FALSE)
-	{
-		clear_favorites (eb);
-	}
-
-	g_signal_connect (history, "visited",
-			  G_CALLBACK (history_site_visited_cb), eb);
-	g_signal_connect (history, "cleared",
-			  G_CALLBACK (history_cleared_cb), eb);
-	g_signal_connect (history, "redirect",
-			  G_CALLBACK (redirect_cb), eb);
-	g_signal_connect (history, "icon-updated",
+#ifdef HAVE_WEBKIT2
+	/* TODO: Favicons */
+#else
+	WebKitFaviconDatabase *favicon_database;
+	
+	favicon_database = webkit_get_favicon_database ();
+	g_signal_connect (favicon_database, "icon-loaded",
 			  G_CALLBACK (icon_updated_cb), eb);
+#endif
 }
 
 static void
@@ -512,7 +296,6 @@ update_bookmark_keywords (EphyBookmarks *eb, EphyNode *bookmark)
 		kid = g_ptr_array_index (children, i);
 
 		if (kid != eb->priv->notcategorized && 
-		    kid != eb->priv->favorites &&
 		    kid != eb->priv->bookmarks &&
 #ifdef ENABLE_ZEROCONF
 		    kid != eb->priv->local &&
@@ -580,7 +363,6 @@ bookmark_is_categorized (EphyBookmarks *eb, EphyNode *bookmark)
 		kid = g_ptr_array_index (children, i);
 
 		if (kid != eb->priv->notcategorized && 
-		    kid != eb->priv->favorites &&
 		    kid != eb->priv->bookmarks &&
 #ifdef ENABLE_ZEROCONF
 		    kid != eb->priv->local &&
@@ -1177,9 +959,6 @@ ephy_bookmarks_init (EphyBookmarks *eb)
 	/* Translators: this topic contains all bookmarks */
 	const char *bk_all = C_("bookmarks", "All");
 
-	/* Translators: this topic contains the most used bookmarks */
-	const char *bk_most_visited = C_("bookmarks", "Most Visited");
-
 	/* Translators: this topic contains the not categorized
 	   bookmarks */
 	const char *bk_not_categorized = C_("bookmarks", "Not Categorized");
@@ -1231,19 +1010,6 @@ ephy_bookmarks_init (EphyBookmarks *eb)
 	ephy_node_add_child (eb->priv->keywords,
 			     eb->priv->bookmarks);
 
-	/* Favorites */
-	eb->priv->favorites = ephy_node_new_with_id (db, FAVORITES_NODE_ID);
-	
-	
-	ephy_node_set_property_string (eb->priv->favorites,
-				       EPHY_NODE_KEYWORD_PROP_NAME,
-				       bk_most_visited);
-	
-	ephy_node_set_property_int (eb->priv->favorites,
-				    EPHY_NODE_KEYWORD_PROP_PRIORITY,
-				    EPHY_NODE_SPECIAL_PRIORITY);
-	ephy_node_add_child (eb->priv->keywords, eb->priv->favorites);
-
 	/* Not categorized */
 	eb->priv->notcategorized = ephy_node_new_with_id (db, BMKS_NOTCATEGORIZED_NODE_ID);
 	
@@ -1320,7 +1086,6 @@ ephy_bookmarks_init (EphyBookmarks *eb)
 			 G_SETTINGS_BIND_GET);
 
 	ephy_setup_history_notifiers (eb);
-	ephy_bookmarks_update_favorites (eb);
 }
 
 static void
@@ -1342,7 +1107,6 @@ ephy_bookmarks_finalize (GObject *object)
 
 	ephy_node_unref (priv->bookmarks);
 	ephy_node_unref (priv->keywords);
-	ephy_node_unref (priv->favorites);
 	ephy_node_unref (priv->notcategorized);
 	ephy_node_unref (priv->smartbookmarks);
 
@@ -1409,8 +1173,12 @@ ephy_bookmarks_add (EphyBookmarks *eb,
 		    const char *title,
 		    const char *url)
 {
-	EphyHistory *history;
 	EphyNode *bm;
+#ifdef HAVE_WEBKIT2
+	/* TODO: Favicons */
+#else
+	WebKitFaviconDatabase *favicon_database;
+#endif
 
 	bm = ephy_node_new (eb->priv->db);
 
@@ -1425,16 +1193,21 @@ ephy_bookmarks_add (EphyBookmarks *eb,
 	}
 	ephy_node_set_property_string (bm, EPHY_NODE_BMK_PROP_TITLE, title);
 
-	history = EPHY_HISTORY (ephy_embed_shell_get_global_history (embed_shell));
-	if (history != NULL)
+#ifdef HAVE_WEBKIT2
+	/* TODO: Favicons */
+#else
+	favicon_database = webkit_get_favicon_database ();
+	if (favicon_database != NULL)
 	{
-		const char *icon = ephy_history_get_icon (history, url);
+		char *icon = webkit_favicon_database_get_favicon_uri (favicon_database, url);
 		if (icon != NULL)
 		{
 			ephy_node_set_property_string
 				(bm, EPHY_NODE_BMK_PROP_ICON, icon);
+			g_free (icon);
 		}
 	}
+#endif
 
 	update_has_smart_address (eb, bm, url);
 	update_bookmark_keywords (eb, bm);
@@ -1773,10 +1546,6 @@ ephy_bookmarks_get_topic_uri (EphyBookmarks *eb,
 	{
 		uri = g_strdup ("topic://Special/NotCategorized");
 	}
-	else if (ephy_bookmarks_get_favorites (eb) == node)
-	{
-		uri = g_strdup ("topic://Special/Favorites");
-	}
 #ifdef ENABLE_ZEROCONF
 	else if (ephy_bookmarks_get_local (eb) == node)
 	{
@@ -1825,10 +1594,6 @@ ephy_bookmarks_find_keyword (EphyBookmarks *eb,
 	{
 		return ephy_bookmarks_get_not_categorized (eb);
 	}
-	else if (strcmp (name, "topic://Special/Favorites") == 0)
-	{
-		return ephy_bookmarks_get_favorites (eb);
-	}
 #ifdef ENABLE_ZEROCONF
 	else if (strcmp (name, "topic://Special/Local") == 0)
 	{
@@ -1944,17 +1709,6 @@ ephy_bookmarks_get_bookmarks (EphyBookmarks *eb)
 	return eb->priv->bookmarks;
 }
 
-/**
- * ephy_bookmarks_get_favorites:
- *
- * Return value: (transfer none):
- **/
-EphyNode *
-ephy_bookmarks_get_favorites (EphyBookmarks *eb)
-{
-	return eb->priv->favorites;
-}
-
 /**
  * ephy_bookmarks_get_local:
  *
