@@ -26,6 +26,7 @@
 #include "ephy-embed-prefs.h"
 #include "ephy-file-helpers.h"
 #include "ephy-private.h"
+#include "ephy-profile-utils.h"
 #include "ephy-session.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
@@ -245,6 +246,7 @@ main (int argc,
   EphyStartupFlags startup_flags;
   EphyEmbedShellMode mode;
   int status;
+  EphyFileHelpersFlags flags;
 
 #ifdef ENABLE_NLS
   /* Initialize the i18n stuff */
@@ -387,10 +389,19 @@ main (int argc,
   /* Work-around Flash Player crash */
   g_setenv ("XLIB_SKIP_ARGB_VISUALS", "1", FALSE);
 
+  /* TODO: we want to migrate each WebApp profile too. */
+  if (!private_instance && !application_mode)
+    ephy_profile_utils_do_migration (-1, FALSE);
+
   /* Start our services */
-  if (!ephy_file_helpers_init (profile_directory,
-                               private_instance || application_mode,
-                               keep_temp_directory || profile_directory,
+  flags = EPHY_FILE_HELPERS_ENSURE_EXISTS;
+
+  if (private_instance || application_mode)
+    flags |= EPHY_FILE_HELPERS_PRIVATE_PROFILE;
+  if (keep_temp_directory || profile_directory)
+    flags |= EPHY_FILE_HELPERS_KEEP_TEMP_DIR;
+
+  if (!ephy_file_helpers_init (profile_directory, flags,
                                &error)) {
     show_error_message (&error);
     exit (1);
@@ -400,7 +411,7 @@ main (int argc,
                                           EPHY_PREFS_LOCKDOWN_ARBITRARY_URL);
 
   if (arguments != NULL && arbitrary_url) {
-    g_print ("URL loading is locked dow n\n");
+    g_print ("URL loading is locked down.\n");
     exit (1);
   }
 
@@ -431,8 +442,6 @@ main (int argc,
     exit (0);
   }
 
-  ephy_file_load_accels ();
-
   /* Now create the shell */
   if (private_instance)
     mode = EPHY_EMBED_SHELL_MODE_PRIVATE;
@@ -489,7 +498,6 @@ main (int argc,
   if (notify_is_initted ())
     notify_uninit ();
 
-  ephy_file_save_accels ();
   ephy_state_save ();
   ephy_embed_prefs_shutdown ();
   ephy_settings_shutdown ();
