@@ -32,7 +32,11 @@
 #include "ephy-window.h"
 #include "ephy-embed.h"
 
+#ifdef HAVE_WEBKIT2
+#include <webkit2/webkit2.h>
+#else
 #include <webkit/webkit.h>
+#endif
 #include <glib-object.h>
 #include <glib.h>
 
@@ -51,6 +55,31 @@ typedef struct _EphyShellPrivate  EphyShellPrivate;
 
 extern EphyShell *ephy_shell;
 
+/**
+ * EphyNewTabFlags:
+ * @EPHY_NEW_TAB_HOME_PAGE: loads the home page in the new tab.
+ * @EPHY_NEW_TAB_NEW_PAGE: legacy synonym for @EPHY_NEW_TAB_HOME_PAGE.
+ * @EPHY_NEW_TAB_OPEN_PAGE: opens the provided network-request.
+ * @EPHY_NEW_TAB_FULLSCREEN_MODE: calls gtk_window_fullscreen on the
+ *        parent window of the new tab.
+ * @EPHY_NEW_TAB_DONT_SHOW_WINDOW: do not show the window where the new
+ *        tab is attached.
+ * @EPHY_NEW_TAB_APPEND_LAST: appends the new tab at the end of the
+ *        notebook.
+ * @EPHY_NEW_TAB_APPEND_AFTER: appends the new tab right after the
+ *        current one in the notebook.
+ * @EPHY_NEW_TAB_JUMP: jumps to the new tab immediately.
+ * @EPHY_NEW_TAB_IN_NEW_WINDOW: creates the new tab in a new window.
+ * @EPHY_NEW_TAB_IN_EXISTING_WINDOW: creates the new tab in the current
+ *        active window, if there is none, creates a window.
+ * @EPHY_NEW_TAB_FROM_EXTERNAL: tries to open the new tab in the current
+ *        active tab if it is currently not loading anything and is
+ *        blank.
+ * @EPHY_NEW_TAB_DONT_COPY_HISTORY: do not copy the back-forward history
+ *        from the current active tab to the new one.
+ *
+ * Controls how new tabs/windows are created and handled.
+ */
 typedef enum {
   /* Page types */
   EPHY_NEW_TAB_HOME_PAGE    = 1 << 0,
@@ -65,6 +94,8 @@ typedef enum {
   EPHY_NEW_TAB_APPEND_LAST  = 1 << 7,
   EPHY_NEW_TAB_APPEND_AFTER = 1 << 8,
   EPHY_NEW_TAB_JUMP   = 1 << 9,
+
+  /* Where */
   EPHY_NEW_TAB_IN_NEW_WINDOW  = 1 << 10,
   EPHY_NEW_TAB_IN_EXISTING_WINDOW = 1 << 11,
 
@@ -118,7 +149,11 @@ EphyEmbed      *ephy_shell_new_tab                      (EphyShell *shell,
 EphyEmbed      *ephy_shell_new_tab_full                 (EphyShell *shell,
                                                          EphyWindow *parent_window,
                                                          EphyEmbed *previous_embed,
+#ifdef HAVE_WEBKIT2
+                                                         WebKitURIRequest *request,
+#else
                                                          WebKitNetworkRequest *request,
+#endif
                                                          EphyNewTabFlags flags,
                                                          EphyWebViewChrome chrome,
                                                          gboolean is_popup,
