@@ -268,7 +268,6 @@ window_cmd_file_open (GtkAction *action,
 	dialog = ephy_file_chooser_new (_("Open"),
 					GTK_WIDGET (window),
 					GTK_FILE_CHOOSER_ACTION_OPEN,
-					EPHY_PREFS_STATE_OPEN_DIR,
 					EPHY_FILE_FILTER_ALL_SUPPORTED);
 
 	g_signal_connect (dialog, "response",
@@ -280,7 +279,7 @@ window_cmd_file_open (GtkAction *action,
 static char *
 get_suggested_filename (EphyWebView *view)
 {
-	char *suggested_filename;
+	char *suggested_filename = NULL;
 	const char *mimetype;
 #ifdef HAVE_WEBKIT2
 	WebKitURIResponse *response;
@@ -308,9 +307,21 @@ get_suggested_filename (EphyWebView *view)
 	}
 	else
 	{
-		SoupURI *soup_uri = soup_uri_new (webkit_web_resource_get_uri (web_resource));
-		suggested_filename = g_path_get_basename (soup_uri->path);
-		soup_uri_free (soup_uri);
+#ifdef HAVE_WEBKIT2
+		/* TODO: suggested filename */
+#else
+		WebKitNetworkResponse *response;
+
+		response = webkit_web_frame_get_network_response (frame);
+		suggested_filename = g_strdup (webkit_network_response_get_suggested_filename (response));
+#endif
+
+		if (!suggested_filename)
+		{
+			SoupURI *soup_uri = soup_uri_new (webkit_web_resource_get_uri (web_resource));
+			suggested_filename = g_path_get_basename (soup_uri->path);
+			soup_uri_free (soup_uri);
+		}
 	}
 
 	return suggested_filename;
@@ -331,7 +342,6 @@ window_cmd_file_save_as (GtkAction *action,
 	dialog = ephy_file_chooser_new (_("Save"),
 					GTK_WIDGET (window),
 					GTK_FILE_CHOOSER_ACTION_SAVE,
-					EPHY_PREFS_STATE_SAVE_DIR,
 					EPHY_FILE_FILTER_NONE);
 
 	gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
@@ -1312,13 +1322,13 @@ window_cmd_help_about (GtkAction *action,
 	g_key_file_free (key_file);
 
 #ifdef HAVE_WEBKIT2
-	comments = g_strdup_printf (_("Lets you view web pages and find information on the internet.\n"
+	comments = g_strdup_printf (_("A simple, clean, beautiful view of the web.\n"
 				      "Powered by WebKit %d.%d.%d"),
 				    webkit_get_major_version (),
 				    webkit_get_minor_version (),
 				    webkit_get_micro_version ());
 #else
-	comments = g_strdup_printf (_("Lets you view web pages and find information on the internet.\n"
+	comments = g_strdup_printf (_("A simple, clean, beautiful view of the web.\n"
 	                              "Powered by WebKit %d.%d.%d"),
 	                            webkit_major_version (),
 	                            webkit_minor_version (),
